
/*
 *      This is a sample DECODER for PacketView. It decodes IP and ARP packets.
 *
 *      Copyright (c) 1992, KLOS Technologies, Inc.
 */

#include "structs.h"
#include "ip.h"

extern unsigned int _dgroup;
char *sprintf();

void format_ip_line();
void format_ip();
void format_arp_line();
void format_arp();

/*                                                                            */
/*      This is a multiple protocol decoder.  It supports both DOD IP         */
/*      and ARP.  Note how they are chained in the init() routine.  The       */
/*      last entry in the chain should contain a NULL next pointer.           */
/*                                                                            */
/*               PPP packet type goes here ============================\\     */
/*                                                                     ||     */
/*               ARCnet packet type goes here   ================\\     ||     */
/*                                                              ||     ||     */
/*         802.2 SAP type goes here ======================\\    ||     ||     */
/*                                                        ||    ||     ||     */
/*     Ethernet type goes here  ===================\\     ||    ||     ||     */
/*                                                 ||     ||    ||     ||     */
/*     Protocol Name goes here =======\\           ||     ||    ||     ||     */
/*                                    ||           ||     ||    ||     ||     */
/*                                    VV          VVVV    VV    VV    VVVV    */

struct protocol ip_protocol =  { 0, "Demo IP",  0x0800, 0x06, 0xf0, 0x0000, 0, format_ip_line, format_ip };
struct protocol arp_protocol = { 0, "Demo ARP", 0x0806, 0x00, 0xf1, 0x0000, 0, format_arp_line, format_arp };

char yes[] = "yes";
char no[] = "no";

char *well_known_protocols[] =
{   "ICMP", "GGP", "TCP", "EGP", "IGP", "CHAOS", "UDP", "TP4"
};

byte protocol_lookup[] =
{   1, 3, 6, 8, 9, 16, 17, 29
};

char *hardware_types[] =
{   "Ethernet (10MB)",
    "Ethernet (3MB)",
    "Amateur Radio AX.25",
    "Proteon PROnet Rings",
    "CHAOSnet",
    "IEEE 802",
    "ARCNET"
};

struct protocol *init()
{
    ip_protocol.dgroup = _dgroup;
    arp_protocol.dgroup = _dgroup;

    ip_protocol.next = &arp_protocol;
    return (&ip_protocol);
}

void format_arp(arp, length)
    struct arp_header *arp;
    int length;
{   int i, j;

    set_color(LTGREEN, YELLOW);

    printf("DEMO DoD ARP:\n");
    i = swap(arp->type);
    if ((i > 1) && (i < 7))
        printf("Hardware type = %s\n", hardware_types[i-1]);
    else
        printf("Hardware type = %04x\n", i);
    printf("Protocol = %04x\n", swap(arp->protocol));
    printf("Node address length = %d, Host address length = %d\n",
                arp->node_len, arp->host_len);
    i = swap(arp->operation);
    if ((i < 1) || (i > 2))
    {   printf("Operation = UNKNOWN (%d)\n", i);
        return;
    }
    printf("Operation = %s\n", (i == 1) ? "REQUEST" : "REPLY");
    printf("Source node address: %n  Source host address: %i\n", arp->source_node_addr, dswap(arp->source_host_id));
    if (i == 1)
        printf("Target node address: UNKNOWN       Target host address: %i\n", dswap(arp->target_host_id));
    else
        printf("Target node address: %n  Target host address: %i\n", arp->target_node_addr, dswap(arp->target_host_id));
}

void format_ip(packet, length)
    byte *packet;
    int length;
{   int i, j, k;
    struct ip_header *ip = (struct ip_header *)packet;

    set_color(LTGREEN, WHITE);

    printf("DEMO DoD IP:\n");
    printf("IP version: %d  IP header length: %d (32-bit words)\n", ((ip->version_length&0xf0)>>4), (ip->version_length&0x0f));
    printf("Type of service: %02x\n", ip->type_of_service);
    printf("Packet length: %04x  Packet ID: %04x\n", swap(ip->length), swap(ip->id));
    i = swap(ip->fragment_offset);
    if (i&0x8000)
        printf("Don't fragment\n");
    else
        printf("More fragments: %s  Fragment offset: %04x\n",
                (i&0x4000) ? yes : no, (i&0x3fff));
    i = ip->protocol;
    for (j=0; j<sizeof(protocol_lookup); j++)
        if (protocol_lookup[j] == i)
            break;
    if (j != sizeof(protocol_lookup))
        printf("Time-to-live: %d  Protocol: %s  Header checksum: %04x\n", ip->time_to_live,
                     well_known_protocols[j], swap(ip->checksum));
    else
        printf("Time-to-live: %d  Protocol: %d  Header checksum: %04x\n", ip->time_to_live,
                     i, swap(ip->checksum));
    printf("Source host id:      %i\n", dswap(ip->source_host_id));
    printf("Destination host id: %i\n", dswap(ip->destination_host_id));

    i = (ip->version_length&0x0f)*4;
    if (length <= i)
        return;

    packet += i;
    length -= i;

    printf("\n");

    switch (ip->protocol)
    {
    case 83: /* Vines IP */
        decode_packet(packet, length, 0xff00);
        break;

    default:
        format_raw("IP Data:", packet, length);
        break;
    }
}

void format_arp_line(b, arp, length)
    char *b;
    struct arp_header *arp;
    int length;
{
    set_color(LTGREEN, YELLOW);
    b = sprintf(b, "DEMO DoD ARP: (%04x) ", swap(arp->protocol));

    switch (swap(arp->operation))
    {
    case 1:
        sprintf(b, "REQUEST from %i for %i", dswap(arp->source_host_id), dswap(arp->target_host_id));
        break;

    case 2:
        sprintf(b, "REPLY from %i to %i", dswap(arp->source_host_id), dswap(arp->target_host_id));
        break;

    default:
        sprintf(b, "UNKNOWN");
        break;
    }
}

void format_ip_line(b, packet, length)
    char *b;
    byte *packet;
    int length;
{   int i, j, k;
    struct ip_header *ip = (struct ip_header *)packet;

    set_color(LTGREEN, WHITE);
    b = sprintf(b, "DEMO DoD IP: %i -> %i ", dswap(ip->source_host_id), dswap(ip->destination_host_id));

    i = ip->protocol;
    for (j=0; j<sizeof(protocol_lookup); j++)
        if (protocol_lookup[j] == i)
            break;
    if (j != sizeof(protocol_lookup))
        b = sprintf(b, "%s: ", well_known_protocols[j]);
    else
        b = sprintf(b, "%d: ", i);
}

